﻿using D3StatsClient.D3MemoryAPI;
using D3StatsClient.Model;
using D3StatsClient.View;
using D3StatsClient.ViewModel;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Threading;

namespace D3StatsClient.Commands
{
    public class StartRecordingSession : CommandBase
    {
        public override bool CanExecute(object parameter)
        {
            var vm = ViewModelManager<MainWindowViewModel>.ViewModel;

            if ((Properties.Settings.Default.EnableMultiProcessSupport 
                && vm.IsSessionLoaded) 
                && !vm.IsRecording
                && !vm.IsContinuedSession)
            {
                return true;
            }

            if (vm.IsRecording || !vm.IsSessionLoaded)
            {
                return false;
            }

            if (vm.IsContinuedSession)
            {
                return false;
            }

            if (vm.DiabloGameIsStarted)
            {
                return true;
            }

            return false;
        }

        public override void Execute(object parameter)
        {
            var vm = ViewModelManager<MainWindowViewModel>.ViewModel;

            //display process select window if multi-process is selected
            if (Properties.Settings.Default.EnableMultiProcessSupport)
            {
                ProcessPickerWindow w = new ProcessPickerWindow();
                w.WindowStartupLocation = System.Windows.WindowStartupLocation.CenterScreen;
                w.ShowDialog();

                var ppvm = ViewModelManager<ProcessPickerWindowViewModel>.ViewModel;

                if (ppvm != null)
                {

                    if (ppvm.SelectedItem == null)
                    {
                        return;
                    }

                    vm.Process = ppvm.SelectedItem;
                }
            }

            var itemFinder = new ItemFinder();

            //Start recording data.
            vm.SetStatus(true);
            vm.IsRecording = true;
            vm.IsSessionLoaded = true;

            vm.sw = new Stopwatch();
            vm.sw.Start();

            DispatcherTimer timer = new DispatcherTimer(new TimeSpan(0, 0, 0, 1), DispatcherPriority.Normal, delegate
            {
                if (vm.IsRecording)
                {
                    var provider = vm.provider as XMLStatsProvider;
                    TimeSpan previoustime = TimeSpan.Zero;

                    if (provider.XMLData.Element("items").Attribute("played") != null)
                    {
                        previoustime = TimeSpan.Parse(provider.XMLData.Element("items").Attribute("played").Value);
                    }

                    vm.SessionTime = previoustime == TimeSpan.Zero ? vm.GetTimeSpanFormattedString(vm.sw.Elapsed) : vm.GetTimeSpanFormattedString((vm.sw.Elapsed + previoustime));
                }

            }, this._currentDispatcher);

            vm.Status = "Current logging item drops...";
        }

        
    }
}
