﻿using D3StatsClient.Model;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;

namespace D3StatsClient.D3MemoryAPI
{
    class ItemFinder : IDisposable
    {
        private DiabloIII d3;
        private List<ACDActor> IgnoreList;
        private List<string> examptions = new List<string>();

        public ItemFinder()
        {
            d3 = new DiabloIII();
            Init();
        }

        public ItemFinder(int processID)
        {
            d3 = new DiabloIII(processID);
            IgnoreList = new List<ACDActor>();
            Init();
        }

        private void Init()
        {
            IgnoreList = new List<ACDActor>();

            examptions.Add("HealthGlobe");
            examptions.Add("Gold");
            examptions.Add("Potion");
            examptions.Add("Ruby");
            examptions.Add("Emerald");
            examptions.Add("Amethyst");
            examptions.Add("Topaz");
            examptions.Add("Tome");
            examptions.Add("Lore_Book");
            examptions.Add("Page");
            examptions.Add("Cow_Bone");
            examptions.Add("Mushroom");
            examptions.Add("Blacksmith");
        }

        public ACDActor Player
        {
            get
            {
                return d3.Player();
            }
        }

        public ACDActor Follower
        {
            get
            {
                return (from l in d3.GetACDActors() where l.Name != null && l.Name.ToLower().Contains("hireling") select l).FirstOrDefault<ACDActor>();
            }
        }

        public int LevelArea
        {
            get
            {
                return d3.CurrentLevelArea();

            }
        }

        public ACDActor FindBestItem(IStatsProvider provider)
        {
            if (d3.Valid == false)
            {
                return null;
            }
            
            ACDActor player = d3.Player();

            if (player == null || player.IsValid() == false)
            {
                return null;
            }

            foreach (ACDActor a in d3.GetACDActors()) //replace getacdactors
            {
                if (a.IsValid() == false)
                {
                    continue;
                }

                if (a.ACDType == ACDType.Item)
                {
                    Item i = new Item(a);

                    //check if we already recorded this drop.
                    if (provider.ContainsDrop(i.Seed))
                    {
                        continue;
                    }

                    //Make sure this is not part of our exampt items.
                    if ((from l in examptions where i.Name.ToLower().IndexOf(l.ToLower()) >= 0 select l).Count() > 0)
                    {
                        continue;
                    }

                    return a;
                }

            }

            return null;
        }

        public void Dispose()
        {
            d3.Dispose();
        }
    }
}
