﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace D3StatsClient.D3MemoryAPI
{
    class SNOReader
    {
        private const int defptr = 0x10;
        private const int defcount = 0x10C;
        private const int deflink = 0x148;

        private Dictionary<int, int> data;

        public SNOReader(MemoryManager reader, int address)
        {
            data = new Dictionary<int, int>();

            int ptr = reader.ReadInt(address, defptr);
            int count = reader.ReadInt(ptr + defcount);
            int curOff = reader.ReadInt(ptr + deflink) + 0xC;

            for (int i = 0; i <= 4096; i++)
            {
                int curSNOoff = reader.ReadInt(curOff);
                int curSNOid = reader.ReadInt(curSNOoff);

                if (curSNOoff == 0 && curSNOid == 0)
                    break;

                data.Add(curSNOid, curSNOoff);

                curOff += 0x10;
            }
        }

        public int GetAddressFromID(int id)
        {
            if (data.ContainsKey(id))
                return data[id];
            return -1;
        }
    }

    public enum SNOGroup
    {
        Textures = 0x16B24C0,
        ShaderMap = 0x1838268,
        Appearance = 0x16B7470,
        PhysMesh = 0x18D61A8,
        Globals = 0x18AE708,
        Observer = 0x18C6498,
        Scene = 0x18D0838,
        Worlds = 0x18C01D8,
        Actor = 0x18C6CE0,
        AnimSet = 0x18FC690,
        Anim = 0x18D3540,
        StringList = 0x18821A8,
        Power = 0x18EF910,
        MarkerSet = 0x18F0268,
        Hero = 0x18ED110,
        GameBalance = 0x18851A0,
        Encounter = 0x18F2238,
        Adventure = 0x19010C8,
        Conversation = 0x18C8740,
        ConversationList = 0x19006E0,
        FlagSet = 0x1900300,
        SceneGroup = 0x18D2A38,
        LevelArea = 0x18BDEC0,
        Quest = 0x18B9E98,
        QuestRange = 0x18D04E0,
        SkillKit = 0x18B86F8,
        Monster = 0x18CC080,
        Recipe = 0x18EF5F0,
        TimedEvent = 0x19001E0,
        Act = 0x18B91F8,
        Lore = 0x18FAE70,
        Condition = 0x18FE438,
        Account = 0x18EBF08,//not updated below this
        TreasureClass = 0x18F9E38,
        BossEncounter = 0x18F7B30,
        Tutorial = 0x18CB648,
        Accolade = 0x18F7A10,
        AmbientSound = 0x186CBA8,
        Physics = 0x1862E68,
        Explosion = 0x186DE58,
        Surface = 0x186AB48,
        EffectGroup = 0x186A420,
        Cloth = 0x186E468,
        Trail = 0x186D618,
        Particle = 0x1860930,
        Font = 0x1832778,
        Anim2D = 0x186ED30,
        Shaders = 0x16BB520,
        UI = 0x16BCF68,
        Light = 0x185C630,
        SoundBank = 0x18636A8,
        Sound = 0x1864118,
        Music = 0x18677E0,
        Rope = 0x186B648,
        Weather = 0x185AEA0,
        Shakes = 0x186AE98,
        Vibrations = 0x1950300,
        Material = 0x180F8C0,
        Reverb = 0x1866A28,
        AnimTree = 0x18ED0C0
    }
}
