﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Media;
using System.IO;
using System.Deployment.Application;
using D3StatsClient.D3MemoryAPI;

namespace D3StatsClient.Model
{
    class Sounds
    {
        private static SoundPlayer standard;
        private static SoundPlayer legendary;
        private static SoundPlayer rare;
        private static SoundPlayer crafting;
        private static SoundPlayer magic;

        public static string DefaultSoundLocation = AppDomain.CurrentDomain.BaseDirectory + "Sounds\\sound.wav";


        public static void InitSounds()
        {
            if (File.Exists(DefaultSoundLocation))
            {
                standard = new SoundPlayer(DefaultSoundLocation);
                standard.Load(); 
            }
            else
            {
                return;
            }

            if (File.Exists(Properties.Settings.Default.LegendarySoundFile))
            {
                legendary = new SoundPlayer(Properties.Settings.Default.LegendarySoundFile);
                legendary.Load();
            }
            else
                legendary = standard;

            if (File.Exists(Properties.Settings.Default.RareSoundFile))
            {
                rare = new SoundPlayer(Properties.Settings.Default.RareSoundFile);
                rare.Load();
            }
            else
                rare = standard;

            if (File.Exists(Properties.Settings.Default.CraftingSoundFile))
            {
                crafting = new SoundPlayer(Properties.Settings.Default.CraftingSoundFile);
                crafting.Load();
            }
            else
                crafting = standard;

            if (File.Exists(Properties.Settings.Default.MagicSoundFile))
            {
                magic = new SoundPlayer(Properties.Settings.Default.MagicSoundFile);
                magic.Load();
            }
            else
                magic = standard;
        }

        public static void PlaySound(Sound sound)
        {
            switch (sound)
            {
                case Sound.None: break;
                case Sound.Legendary: legendary.PlaySync(); break;
                case Sound.Rare: rare.PlaySync(); break;
                case Sound.Crafting: crafting.PlaySync(); break;
                case Sound.Magic: magic.PlaySync(); break;
            }
        }

        public static void PlayNotificationSound(ItemQuality qLevel, int iLvl)
        {
            try
            {

                switch (qLevel)
                {
                    case ItemQuality.Magic:

                        if (Properties.Settings.Default.PlayOnMagic)
                        {
                            int minLvl = 0;
                            bool result = int.TryParse(Properties.Settings.Default.MagicNotificationMinLevel, out minLvl);

                            //This meanns that the notification level is all.
                            if (result == false)
                            {
                                Sounds.PlaySound(Sound.Magic);
                            }
                            else
                            {
                                if (minLvl <= iLvl)
                                {
                                    Sounds.PlaySound(Sound.Magic);
                                }
                            }
                        }

                        break;


                    case ItemQuality.Legendary:

                        if (Properties.Settings.Default.PlayOnLegendary)
                        {
                            int minLvl = 0;
                            bool result = int.TryParse(Properties.Settings.Default.LegendaryNotificationMinLevel, out minLvl);

                            //This meanns that the notification level is all.
                            if (result == false)
                            {
                                Sounds.PlaySound(Sound.Legendary);
                            }
                            else
                            {
                                if (minLvl <= iLvl)
                                {
                                    Sounds.PlaySound(Sound.Legendary);
                                }
                            }
                        }

                        break;
                    
                   
                    case ItemQuality.Rare:

                        if (Properties.Settings.Default.PlayOnRare)
                        {
                            int minLvl = 0;
                            bool result = int.TryParse(Properties.Settings.Default.RareNotificationMinLevel, out minLvl);

                            //This meanns that the notification level is all.
                            if (result == false)
                            {
                                Sounds.PlaySound(Sound.Rare);
                            }
                            else
                            {
                                //Check if the iLevel is one we trigger sound on.
                                if (minLvl <= iLvl)
                                {
                                    Sounds.PlaySound(Sound.Rare);
                                }
                            }
                        }

                        break;

                    case ItemQuality.CraftingPlan:

                        if (Properties.Settings.Default.PlayOnCrafting)
                        {
                            Sounds.PlaySound(Sound.Crafting);
                        }

                        break;
                }
            }
            catch (Exception ex)
            {
                Logging.Logger.Log(string.Format("An error occured in PlayNotificationSound(). The error was {0}", ex.Message));
            }
        }
    }



    public enum Sound
    {
        None,
        Legendary,
        Rare,
        Crafting,
        Magic,
        Goblin
    }
}
